/********************************************
 * UTIA AV CR, v.v.i
 * Signal Processing Department
 * Pod Vodarenskou vezi 4
 * 182 08 Prague
 * Czech Republic
 * web: http://zs.utia.cas.cz
 * contact: sp@utia.cz
 *
 * Filename: i2c_ps.c
 * Date: 17.03.2021
 * Author: Lukas Kohout <kohoutl@utia.cas.cz>
 * Description:
 *
 * Changelog:
 *
 *
 *
 *
 ********************************************/


#include "i2c_ps.h"


/*
 * Initialize PS I2C core
 *
 * @param   *i2c_ps is pointer to I2C instance
 *
 * @return  - XST_SUCCESS
 *          - XST_FAILURE
 */
int i2c_ps_init(i2c_ps_t *i2c_ps)
{
    int status;

    XIicPs_Config *config = &(i2c_ps->inst.Config);

    /*
     * Initialize the IIC driver so that it's ready to use
     * Look up the configuration in the config table,
     * then initialize it.
     */
    config = XIicPs_LookupConfig( i2c_ps->dev_id );
    if ( config == NULL ) return XST_FAILURE;

    status = XIicPs_CfgInitialize( &(i2c_ps->inst), config, config->BaseAddress);
    if ( status != XST_SUCCESS ) return status;


    /*
     * Perform a self-test to ensure that the hardware was built correctly.
     */
    status = XIicPs_SelfTest( &(i2c_ps->inst) );
    if ( status != XST_SUCCESS ) return status;

    /*
     * Set the IIC serial clock rate.
     */
    status = XIicPs_SetSClk( &(i2c_ps->inst), i2c_ps->sclk_hz);
    if ( status != XST_SUCCESS ) return status;

    return XST_SUCCESS;
}


/*
 * TE0BF808 uses TCA9548 8-channel I2C switch
 * This function selects required channel
 *
 * @param   *i2c_ps is pointer to I2C instance
 *
 * @return  - XST_SUCCESS
 *          - XST_FAILURE
 *
 * @note    This routine is for polled mode transfer only.
 */
int ic2_ps_9548_sel(i2c_ps_t *i2c_ps, u8 sel)
{
    int status;

    // Wait until bus is idle to start another transfer.
    while ( XIicPs_BusIsBusy(&(i2c_ps->inst)) ) { }

    // send configuartion byte to 9548 I2C switch
    status = XIicPs_MasterSendPolled( &(i2c_ps->inst), &sel, 1, I2C_PS_9548_SLAVE_ADDR);
    if ( status != XST_SUCCESS ) return XST_FAILURE;

    return XST_SUCCESS;
}


/*
 * I2C Write register
 *
 * @param   *i2c_ps is pointer to I2C instance
 * @param   dev_addr is 7-bit I2C address of the slave
 * @param   reg is 8-bit address of the register to be written
 * @param   data is 8-bit value of the register
 *
 * @return  - XST_SUCCESS
 *          - XST_FAILURE
 *
 * @note    This routine is for polled mode transfer only.
 */
int i2c_ps_write_reg(i2c_ps_t *i2c_ps, u8 dev_addr, u8 reg, u8 data)
{
    int status;

    u8 buff[2];
    buff[0] = reg;
    buff[1] = data;

    // Wait until bus is idle to start another transfer.
    while ( XIicPs_BusIsBusy(&(i2c_ps->inst)) ) { }

    status = XIicPs_MasterSendPolled( &(i2c_ps->inst), buff, 2, dev_addr);
    if ( status != XST_SUCCESS ) return status;

    return XST_SUCCESS;
}


/*
 * I2C Read register
 *
 * @param   *i2c_ps is pointer to I2C instance
 * @param   dev_addr is 7-bit I2C address of the slave
 * @param   reg is 8-bit address of the register to be read
 * @param   data is pointer to received 8-bit register value
 *
 * @return  - XST_SUCCESS
 *          - XST_FAILURE
 *
 * @note    This routine is for polled mode transfer only.
 */
int i2c_ps_read_reg(i2c_ps_t *i2c_ps, u8 dev_addr, u8 reg, u8 *data)
{
    int status;

//     u8 buff = 1;

    // Wait until bus is idle to start another transfer.
    while ( XIicPs_BusIsBusy(&(i2c_ps->inst)) ) { }

    status = XIicPs_MasterSendPolled( &(i2c_ps->inst), &reg, 1, dev_addr);
    if ( status != XST_SUCCESS ) return XST_FAILURE;

    status = XIicPs_MasterRecvPolled( &(i2c_ps->inst), data, 1, dev_addr);
    if ( status != XST_SUCCESS ) return XST_FAILURE;

    return XST_SUCCESS;
}


/*
 * I2C Write buffer
 *
 * @param   *i2c_ps is pointer to I2C instance
 * @param   dev_addr is 7-bit I2C address of the slave
 * @param   buff is 8-bit pointer of the buffer to be written
 * @param   len is number of bytes
 *
 * @return  - XST_SUCCESS
 *          - XST_FAILURE
 *
 * @note    This routine is for polled mode transfer only.
 */
int i2c_ps_write_buff(i2c_ps_t *i2c_ps, u8 dev_addr, u8 *buff, u32 len)
{
    int status;

    // Wait until bus is idle to start another transfer.
    while ( XIicPs_BusIsBusy(&(i2c_ps->inst)) ) { }

    status = XIicPs_MasterSendPolled( &(i2c_ps->inst), buff, len, dev_addr);
    if ( status != XST_SUCCESS ) return status;

    return XST_SUCCESS;
}


